// CertificateInfo.cpp: implementation of the CCertificateInfo class.
//
//////////////////////////////////////////////////////////////////////

#include "CertificateInfo.h"
#include <vtk_cert.h>
#include <cassert>

/*
#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif
*/
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


CCertificateInfo::CCertificateInfo():m_pCertInfo(NULL)
{
}


CCertificateInfo::CCertificateInfo(const CCertificateInfo& obj)
{
	*this = obj;
}

CCertificateInfo& CCertificateInfo::operator=(const CCertificateInfo& obj)
{
	if (this != &obj)
	{
		// BUGBUG
		// Assignment operator is temporary not available !
		// Support from Toolkit Validator is needed. Possible solutions:
		//  - copy Vtk_CertInfo structure
		//  - increase reference counter in for Vtk_CertInfo structure of obj

		// this->m_pCertInfo = obj.m_pCertInfo;
		throw VString("Assignment of CCertificateInfo is not supported.");
	}
	return *this;
}


CCertificateInfo::~CCertificateInfo()
{
	if (this->m_pCertInfo)
		Vtk_CertInfoDelete(this->m_pCertInfo);
}


// Accessors Functions

long CCertificateInfo::Version() const 
{
	assert(this->m_pCertInfo);

	return m_pCertInfo->version;
}

VString CCertificateInfo::SerialNum() const 
{ 
	assert(this->m_pCertInfo);

	VString strOut = (const char *)m_pCertInfo->serialNum.dPtr;
	return strOut;
}


VString CCertificateInfo::SignatureAlg() const 
{ 
	assert(this->m_pCertInfo);

	VString strOut = (const char *) m_pCertInfo->signatureAlg.dPtr;
	return strOut;
}


VString CCertificateInfo::Issuer() const 
{ 
	assert(this->m_pCertInfo);

	VString strOut = (const char *) m_pCertInfo->issuer.dPtr;
	return strOut;
}


time_t CCertificateInfo::NotBefore() const 
{ 
	assert(this->m_pCertInfo);

	return m_pCertInfo->notBefore;
}


time_t CCertificateInfo::NotAfter() const 
{ 
	assert(this->m_pCertInfo);

	return m_pCertInfo->notAfter;
}


VString CCertificateInfo::Subject() const 
{ 
	assert(this->m_pCertInfo);

	VString strOut = (const char *) m_pCertInfo->subject.dPtr;
	return strOut;
}


VString CCertificateInfo::PublicKeyAlg() const 
{ 
	assert(this->m_pCertInfo);

	VString strOut = (const char *) m_pCertInfo->publicKeyAlg.dPtr;
	return strOut;
}


VString CCertificateInfo::IssuerUniqueId() const 
{
	assert(this->m_pCertInfo);

	VString strOut;
	if (this->m_pCertInfo->issuerUniqueId)
		strOut = (const char *) this->m_pCertInfo->issuerUniqueId->dPtr;

	return strOut;
}

VString CCertificateInfo::SubjectUniqueId() const
{ 
	assert(this->m_pCertInfo);

	VString strOut;
	if (this->m_pCertInfo->subjectUniqueId)
		strOut = (const char *) this->m_pCertInfo->subjectUniqueId->dPtr;

	return strOut;
};


ostream& operator<<(ostream& os, const CCertificateInfo& obj)
{
	time_t startTime = obj.NotBefore();
	time_t endTime = obj.NotAfter();

	os << "Version:              " << obj.Version() << endl; 
	os << "Serial Number:        " << obj.SerialNum() << endl;
	os << "Subject:              " << obj.Subject() << endl;
	os << "Issuer:               " << obj.Issuer() << endl;

	if (startTime > 0)
		os << "Start Date:           " << ctime(&startTime);
	else
		os << "Start Date:           " << "Unset" << endl;

	if (endTime > 0)
		os << "End Date:             " << ctime(&endTime);
	else
		os << "End Date:             " << "Unset" << endl;


	os << "Signature Algorithm:  " << obj.SignatureAlg() << endl;
	os << "Public Key Algorithm: " << obj.PublicKeyAlg() << endl;
	os << "Issuer Unique Id:     " << obj.IssuerUniqueId() << endl;
	os << "Subject Unique Id:    " << obj.SubjectUniqueId() << endl;

	return os;
}
