// ValQueries.cpp: implementation of the CValQueries class.
//
//////////////////////////////////////////////////////////////////////

#include "ValQueries.h"
#include "ProtocolDetails.h"
#include <vtk_valid.h>
#include <cassert>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CValQueries::CValQueries():m_ppQuery(NULL), m_size(0)
{
}

CValQueries::~CValQueries()
{
	if (m_ppQuery)
	{
		// First delete all responses that were set by SetResponse()
		for (int i=0; i<m_size; i++)
		{
			if (m_ppQuery[i]->response.dPtr)
				delete [] m_ppQuery[i]->response.dPtr;
		}

		// Delete Vtk_ValQuery allocated by CValidator::GetQueries()
		Vtk_ValQueriesDelete(m_ppQuery);
	}
}


CValQueries::CValQueries(const CValQueries& obj)
{
	*this = obj;
}

CValQueries& CValQueries::operator=(const CValQueries& obj)
{
	if (this != &obj)
	{
		// BUGBUG
		// Temporary the assignment is not allowed !
		// Support from Toolkit Validator is needed.
		throw VString("Assignment of CValQueries is not supported.");
	}
	return *this;
}


// Accessros for Vtk_ValQuery data
//////////////////////////////////////////////////////////////////////

int CValQueries::Size() const
{
	return this->m_size;
}


enum Vtk_ValidationMech CValQueries::ValidationMech(int nIndex) const
{
	assert(m_ppQuery);

	// Check range
	if (nIndex < 0 || nIndex >= m_size)
		throw VString("Index is out of the range.");

	return m_ppQuery[nIndex]->type;
}


VString CValQueries::Host(int nIndex) const
{
	assert(m_ppQuery);

	// Check range
	if (nIndex < 0 || nIndex >= m_size)
		throw VString("Index is out of the range.");

	VString strOut = m_ppQuery[nIndex]->host;
	return strOut;
}


int CValQueries::Port(int nIndex) const
{
	assert(m_ppQuery);

	// Check range
	if (nIndex < 0 || nIndex >= m_size)
		throw VString("Index is out of the range.");

	return m_ppQuery[nIndex]->port;
}


VString CValQueries::Url(int nIndex) const
{
	assert(m_ppQuery);

	// Check range
	if (nIndex < 0 || nIndex >= m_size)
		throw VString("Index is out of the range.");

	VString strOut = m_ppQuery[nIndex]->url;
	return strOut;
}


void CValQueries::ProtocolDetails(int nIndex, CProtocolDetails& detailsOut) const
{
	assert(m_ppQuery);

	// Check range
	if (nIndex < 0 || nIndex >= m_size)
		throw VString("Index is out of the range.");

	detailsOut.m_details = *(m_ppQuery[nIndex]->protocolDetails);
}


void CValQueries::Request(int nIndex, vector<unsigned char>& requestOut) const
{
	assert(m_ppQuery);

	// Check range
	if (nIndex < 0 || nIndex >= m_size)
		throw VString("Index is out of the range.");

	for (int i=0; i < m_ppQuery[nIndex]->request.len; i++)
		requestOut.push_back(m_ppQuery[nIndex]->request.dPtr[i]);
}


void CValQueries::GetResponse(int nIndex, vector<unsigned char>& responseOut) const
{
	assert(m_ppQuery);

	// Check range
	if (nIndex < 0 || nIndex >= m_size)
		throw VString("Index is out of the range.");

	for (int i=0; i < m_ppQuery[nIndex]->response.len; i++)
		responseOut.push_back(m_ppQuery[nIndex]->response.dPtr[i]);
}


void CValQueries::SetResponse(int nIndex, const vector<unsigned char>& responseIn)
{
	assert(m_ppQuery);

	// Check range
	if (nIndex < 0 || nIndex >= m_size)
		throw VString("Index is out of the range.");

	int nSize = responseIn.size();
	if (nSize)
	{
		// Allocate memory for new response and copy data
		m_ppQuery[nIndex]->response.dPtr = new unsigned char[nSize];
		if (! m_ppQuery[nIndex]->response.dPtr)
			throw VString("Not enough memory.");
	
		for (int i=0; i<nSize; i++)
			m_ppQuery[nIndex]->response.dPtr[i] = responseIn.at(i);


		m_ppQuery[nIndex]->response.len = nSize;

		// BUGBUG - Set data format (VTK_DF_STRING, VTK_DF_DER, VTK_DF_BASE64, VTK_DF_HEX)
	}
}



void CValQueries::SetResponse(int nIndex, const unsigned char* pResponse, int responseSize)
{
	assert(m_ppQuery);

	// Check range
	if (nIndex < 0 || nIndex >= m_size)
		throw VString("Index is out of the range.");

	if (responseSize)
	{
		// Allocate memory for new response and copy data
		m_ppQuery[nIndex]->response.dPtr = new unsigned char[responseSize];
		if (! m_ppQuery[nIndex]->response.dPtr)
			throw VString("Not enough memory.");

		memcpy(m_ppQuery[nIndex]->response.dPtr, pResponse, responseSize);

		m_ppQuery[nIndex]->response.len = responseSize;

		// BUGBUG - Set data format (VTK_DF_STRING, VTK_DF_DER, VTK_DF_BASE64, VTK_DF_HEX)
	}
}



