// ValRespDetails.cpp: implementation of the CValRespDetails class.
//
//////////////////////////////////////////////////////////////////////

#include "ValRespDetails.h"
#include "Extension.h"
#include <cassert>
#include <vtk_cert.h>
#include <vtk_valid.h>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CValRespDetails::CValRespDetails():m_pRespDetails(NULL)
{

}

CValRespDetails::~CValRespDetails()
{
	if (m_pRespDetails)
		Vtk_ValRespDetailsDelete(m_pRespDetails);
}

CValRespDetails::CValRespDetails(const CValRespDetails& obj)
{
	*this = obj;
}

CValRespDetails& CValRespDetails::operator=(const CValRespDetails& obj)
{
	if (this != &obj)
	{
		// BUGBUG
		// Temporary the assignment is not allowed !
		// Support from Toolkit Validator is needed.
		throw VString("Assignment of CValRespDetails is not supported.");
	}
	return *this;
}




////////////////////////////////////////////////////////////////////////
// Accessors


enum Vtk_ValidationMech CValRespDetails::ValidationMech() const
{
	assert(m_pRespDetails);
	return m_pRespDetails->type;
}

Vtk_uint32 CValRespDetails::Version() const
{
	assert(m_pRespDetails);
	return m_pRespDetails->version;
}

VString CValRespDetails::IssuerIdByName() const
{
	assert(m_pRespDetails);

	VString strOut;
	if (m_pRespDetails->issuerIdByName)
		strOut = (char *) m_pRespDetails->issuerIdByName->dPtr;

	return strOut;
}

VString CValRespDetails::IssuerIdByKey() const
{
	assert(m_pRespDetails);

	VString strOut;
	if (m_pRespDetails->issuerIdByKey)
		strOut = (char *) m_pRespDetails->issuerIdByKey->dPtr;

	return strOut;
}

time_t CValRespDetails::IssueTime() const
{
	assert(m_pRespDetails);
	return m_pRespDetails->issueTime;
}


Vtk_uint32 CValRespDetails::MinVersionToRead() const
{
	assert(ValidationMech() == VTK_VM_CRT);
	assert(m_pRespDetails->d.crt);

	return m_pRespDetails->d.crt->minVersionToRead;
}


time_t CValRespDetails::NextUpdate() const
{
	enum Vtk_ValidationMech mech = ValidationMech();
	assert(mech == VTK_VM_CRT || mech == VTK_VM_CRL);

	time_t timeOut;
	if (mech == VTK_VM_CRT)
	{
		assert(m_pRespDetails->d.crt);
		timeOut = m_pRespDetails->d.crt->nextUpdate;
	}
	else /* if (mech == VTK_VM_CRL)  */
	{
		assert(m_pRespDetails->d.crl);
		timeOut = m_pRespDetails->d.crl->nextUpdate;
	}

	return timeOut;
}

time_t CValRespDetails::ValidUntil() const
{
	assert(ValidationMech() == VTK_VM_CRT);
	assert(m_pRespDetails->d.crt);

	return m_pRespDetails->d.crt->validUntil;
}

VString CValRespDetails::HashAlgorithm() const
{
	assert(ValidationMech() == VTK_VM_CRT);
	assert(m_pRespDetails->d.crt);

	VString strOut;
	if (m_pRespDetails->d.crt->hashAlgorithm.dPtr)
		strOut = (char *) m_pRespDetails->d.crt->hashAlgorithm.dPtr;

	return strOut;
}


Vtk_uint32 CValRespDetails::FullTreeLeafCount() const
{
	assert(ValidationMech() == VTK_VM_CRT);
	assert(m_pRespDetails->d.crt);

	return m_pRespDetails->d.crt->fullTreeLeafCount;
}


void CValRespDetails::Print(ostream& os, const CContext& context) const
{
	cout << "Validation Mechanism: ";
	if (ValidationMech() == VTK_VM_CRT)
		cout << "CRT" << endl;
	else if (ValidationMech() == VTK_VM_OCSP)
		cout << "OCSP" << endl;
	else if (ValidationMech() == VTK_VM_CRL)
		cout << "CRL" << endl;
	else
		cout << "Unset" << endl;

	cout << "Version:              " << Version() << endl;
	cout << "Issuer Id By Name:    " << IssuerIdByName() << endl;
	cout << "Issuer Id By Key:     " << IssuerIdByKey() << endl;
	time_t issueTime = IssueTime();
	if (issueTime > 0)
		cout << "Issue Time:           " << ctime(&issueTime);
	else
		cout << "Issue Time:           " << "Unset" << endl;


	if (ValidationMech() == VTK_VM_CRT)
	{
		cout << "Min Version To Read:  " << MinVersionToRead() << endl;
		time_t nextUpdate = NextUpdate();
		if (nextUpdate > 0)
			cout << "Next Update:          " << ctime(&nextUpdate);
		else
			cout << "Next Update:          " << "Unset" << endl;

		time_t validUntil = ValidUntil();
		if (validUntil > 0)
			cout << "Valid Until:          " << ctime(&validUntil);
		else
			cout << "Valid Until:          " << "Unset" << endl;

		cout << "Hash Algorithm:       " << HashAlgorithm() << endl;
		cout << "Full Tree Leaf Count: " << FullTreeLeafCount() << endl;
	}
	else if ( ValidationMech() == VTK_VM_CRL)
	{
		time_t nextUpdate = NextUpdate();
		if (nextUpdate > 0)
			cout << "Next Update:          " << ctime(&nextUpdate);
		else
			cout << "Next Update:          " << "Unset" << endl;
	}

	// Exceptions
	int numExts = 0;
	if (m_pRespDetails->extensions)
		numExts = Vtk_ExtensionsGetCount(context.GetVtkCtxt(), m_pRespDetails->extensions);

	cout << "Number of Exceptions: " << numExts << endl;
}

