// ValRespDetails.h: interface for the CValRespDetails class.
//
//////////////////////////////////////////////////////////////////////
//
// Structures describing detailed revocation information.
// This includes contents of OCSP/CRT responses and CRLs.
//


#if !defined(AFX_VALRESPDETAILS_H__98CB56F1_9DFA_11D3_80E9_00A0C9E2EA8A__INCLUDED_)
#define AFX_VALRESPDETAILS_H__98CB56F1_9DFA_11D3_80E9_00A0C9E2EA8A__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "VString.h"
#include "Certificate.h"
#include "Validator.h"
#include "ValHdl.h"

class CExtensions;

class __exportVTKC CValRespDetails  
{
	// Following friends are only functions that can set CValRespDetails data
	friend void CCertificate::CRLValidate(const CContext& context, Vtk_uint32 *pStatus,
							   CValRespDetails *pRespDetails /*= NULL*/,
							   CValRespSingleCertDetails *pCertDetails/*=NULL*/) const;

	friend void CValidator::GetRevStatus(const CContext& context, const CCertificate& cert,
						      const CCertificate& issuerCert, Vtk_uint32 *pStatus, 
							  CValRespDetails *pRespDetails /*=NULL*/,
							  CValRespSingleCertDetails *pCertDetails /*=NULL*/) const;

	friend void CValHdl::GetRevStatus(const CContext& context, Vtk_uint32 *pStatus, 
							  CValRespDetails *pRespDetails /*=NULL*/,
							  CValRespSingleCertDetails *pCertDetails /*=NULL*/) const;

private:

	// Temporary (until necessary functions from Toolkit are provided)
	// make copy constructor and assign operator not available
	CValRespDetails(const CValRespDetails& obj);
	CValRespDetails& operator=(const CValRespDetails& obj);


public:
	CValRespDetails();
	virtual ~CValRespDetails();

	// Check if structure Vtk_ValRespDetails was initialized
	bool IsNull() const { return m_pRespDetails == NULL ? true : false; }

	// Accessors to Vtk_ValRespDetails data
	enum Vtk_ValidationMech ValidationMech() const;
	Vtk_uint32 Version() const;
	VString IssuerIdByName() const;
	VString IssuerIdByKey() const;
	time_t  IssueTime() const;

	Vtk_uint32 MinVersionToRead() const;
	time_t NextUpdate() const;
	time_t ValidUntil() const;
	VString HashAlgorithm() const;
	Vtk_uint32 FullTreeLeafCount() const;

	// Print CValRespDetails object to standard output 
	void Print(ostream& os, const CContext& context) const;


protected:

	// Detailed revocation information for the entire validation response
	Vtk_ValRespDetails *m_pRespDetails;

};


#endif // !defined(AFX_VALRESPDETAILS_H__98CB56F1_9DFA_11D3_80E9_00A0C9E2EA8A__INCLUDED_)
