// Validator.h: interface for the CValidator class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_VALIDATOR_H__A03B0F70_6719_11D3_80B3_00A0C9E2EA8A__INCLUDED_)
#define AFX_VALIDATOR_H__A03B0F70_6719_11D3_80B3_00A0C9E2EA8A__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "DLLDefs.h"
#include <vtk_defs.h>

class VString;
class CContext;
class CCertificate;
class CCertificateStore;
class CValRespDetails;
class CValRespSingleCertDetails;
class CValHdl;
class CValQueries;
class CExtension;

class __exportVTKC CValidator  
{
private:

	// Temporary (until necessary functions from Toolkit are provided)
	// make copy constructor and assign operator not available
	CValidator(const CValidator& obj);
	CValidator& operator=(const CValidator& obj);

public:
	CValidator();
	CValidator(const CContext& context);
	virtual ~CValidator();

	// Add certificate to the validation query
	void AddCertificate(const CContext& context, const CCertificate& cert,
						const CCertificate& issuerCert, CValHdl *pHdl = NULL);

	void AddCertificate(const CContext& context, const unsigned char *pCertSource, 
		                int certSize, enum Vtk_DataFormat certFormat, 
		                const unsigned char *pIssuerCertSource, int issuerCertSize, 
						enum Vtk_DataFormat issuerCertFormat, CValHdl *pHdl = NULL);


	// Adding certificate chain to the validation
	void AddCertificateChain(const CContext& context, const CCertificate& cert,
							 Vtk_ChainBuildCallback callback = NULL, 
							 void *userHdl = NULL);

	// Sets specific validation protocol and VA information for this validation 
	// operation. The setting provided here will override any Vtk_Ctxt settings.
	void SetVaInfo(const CContext& context, const VString& vaUrl, 
	               enum Vtk_ValidationMech mech, 
				   const CCertificateStore* pCertStore=NULL);


	// Perform validation query
	void Validate(const CContext& context, Vtk_uint32 *pValStatus, 
		          VString *pStrStatus) const;

	// Retrives validation status information for a single certificate.
	void GetRevStatus(const CContext& context, const CCertificate& cert,
					  const CCertificate& issuerCert, Vtk_uint32 *pStatus, 
					  CValRespDetails *pRespDetails =NULL,
					  CValRespSingleCertDetails *pCertDetails =NULL) const;


	// Obtains an array of Validation query messages in order for the
	// application to perform the I/O with the VA itself. The application
	// should set the response field for each query.
	void GetQueries(const CContext& context, CValQueries& valQueries) const;

	// Performs the equivalent of the Vtk_ValidationValidate, but uses
	// the data in the Vtk_ValQuery->response as the source of validation
	// information.
	void ValidateFromQueries(const CContext& context, CValQueries& valQueries,
							Vtk_uint32 *pStatus, VString *pStrStatus);

	// Obtains CValHdl object for certificate pair which was previously
	// added to CValidator. Issuer certificate can be NULL if it is in context
	void GetValHdl(const CContext& context, const CCertificate& cert,
	               const CCertificate *pIssuerCert, CValHdl *pHdl);

	// Sets extension in Va request for whole validation
	void AddReqExt(const CContext& context, const CExtension& ext);


	// Sets extension in Va equest for particular certificate
	// Certificate is specified by user cert, issuer cert pair
	// Issuer certificate can be NULL, if it was already added ino context
	void AddReqExtForSingleCert(const CContext& context, const CCertificate& cert, 
		                        const CCertificate *pIssuerCert, const CExtension& ext);


protected:
	
	// Validation query structure
	Vtk_Validation *m_pValidation;

};

#endif // !defined(AFX_VALIDATOR_H__A03B0F70_6719_11D3_80B3_00A0C9E2EA8A__INCLUDED_)
